// src/use-isomorphic-effect.ts
import { useLayoutEffect, useEffect } from "react";
var useIsomorphicEffect = typeof window === "undefined" ? useEffect : useLayoutEffect;

// src/use-previous.ts
import { useEffect as useEffect2, useRef } from "react";
function usePrevious(state) {
  const ref = useRef();
  useEffect2(() => {
    ref.current = state;
  });
  return ref.current;
}

// src/use-previous-distinct.ts
import { useRef as useRef2 } from "react";
function usePreviousDistinct(value, compare = (prev, next) => prev === next) {
  const mountedRef = useRef2(false);
  const previousRef = useRef2();
  const currentRef = useRef2(value);
  if (mountedRef.current && !compare(currentRef.current, value)) {
    previousRef.current = currentRef.current;
    currentRef.current = value;
  }
  mountedRef.current = true;
  return previousRef.current;
}

// src/use-business-state.ts
import { useSyncExternalStore, useCallback, useRef as useRef3 } from "react";
import { deepEqual } from "fast-equals";
var SEPARATOR = "__@.@__";
function useBusinessState(emitter, events, evalFn, args) {
  const eventStr = events.join(SEPARATOR);
  const subscribe = useCallback(
    (listener) => {
      const types = eventStr.split(SEPARATOR);
      types.forEach((type) => {
        emitter.on(type, listener);
      });
      return () => {
        types.forEach((type) => {
          emitter.off(type, listener);
        });
      };
    },
    [emitter, eventStr]
  );
  const evalFnRef = useRef3(evalFn);
  evalFnRef.current = evalFn;
  const argsRef = useRef3(args);
  argsRef.current = args;
  const currentValueRef = useRef3();
  const getSnapshot = useCallback(() => {
    const nextValue = evalFnRef.current(emitter, argsRef.current);
    const currentValue = currentValueRef.current;
    currentValueRef.current = deepEqual(nextValue, currentValue) ? currentValue : nextValue;
    return currentValueRef.current;
  }, [emitter]);
  return useSyncExternalStore(subscribe, getSnapshot);
}

// src/use-business-event.ts
import { useRef as useRef4 } from "react";
var SEPARATOR2 = "__@.@__";
function useBusinessEvent(emitter, events, listener, args) {
  const listenerRef = useRef4(listener);
  listenerRef.current = listener;
  const argsRef = useRef4(args);
  argsRef.current = args;
  const eventStr = events.join(SEPARATOR2);
  useIsomorphicEffect(() => {
    const listenerFn = () => {
      listenerRef.current(emitter, argsRef.current);
    };
    const types = eventStr.split(SEPARATOR2);
    types.forEach((type) => {
      emitter.on(type, listenerFn);
    });
    return () => {
      types.forEach((type) => {
        emitter.off(type, listenerFn);
      });
    };
  }, [emitter, eventStr]);
}

// src/use-business.ts
var deprecated = (fn, message) => {
  let warned = false;
  const wrapped = (...args) => {
    if (!warned) {
      console.warn(message);
      warned = true;
    }
    return fn(...args);
  };
  return wrapped;
};
var useBusiness = deprecated(
  useBusinessState,
  "useBusiness is deprecated and will be removed in the next major version, please use useBusinessState instead."
);
export {
  useBusiness,
  useBusinessEvent,
  useBusinessState,
  useIsomorphicEffect,
  usePrevious,
  usePreviousDistinct
};
